/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import org.w3c.dom.*;


import java.util.*;

/**
 * Children (besides the inherited):<br>
 *   value, value...<br>
 *   option, option...<br>
 */
public abstract class A_MultiValueList extends A_Field {

	private List values;
	private List options;

	public A_MultiValueList(Form form) {
		super(form);
	}
	
	public A_MultiValueList(Form form, Node field) {
		super(form, field);
	}
	
	public int cardinality() {
		return I_Field.CARDINALITY_MULTIPLE;
	}
	
	/**
	 * Look for relevant sub-elements
	 * @param child
	 * @param childType
	 * @param childName
	 * @return true if child is considered handled, false if it didn't cause any changes
	 */
	public boolean handleChildNode(Node child, short childType, String childName)
	{
		if(super.handleChildNode(child, childType, childName))
			return true;

		if(childType == Node.ELEMENT_NODE) {
			if (childName.equals(Form.FBTAG_VALUE)) {
				addValue(Form.collectText(child));
				return true;
			}
			else if (childName.equals(Form.FBTAG_OPTION)) {
				Option opt = new Option(form, child);
				addOption(opt);
				return true;
			} 
		}
		return false;
	}

	//
	// accessors
	//
	
	public String getValue() {
		if(values == null || values.size() == 0)
			return null;
		else
			return (String)values.get(0);
	}
	
	/**
	 * Sets the values to this one value (throwing away any previously set values).<br>
	 * If you need to add a value to the existing set of values use <code>addValue()</code>.
	 * 
	 * @param value
	 */
	public void setValue(String value){
		if(values == null)
			values = new ArrayList();
		else
			values.clear();
		values.add(value);
	}
	
	public List getValues() {
		return values;
	}
	
	public void setValues(List l){
		this.values = l;
	}

	public void addValue(String s) {
		if(values == null)
			values = new ArrayList();
		values.add(s);
	}
	
	public void removeValue(String s) {
		if(values != null)
			values.remove(s);
	}
	
	public void setValues(String[] values) {
		if(values == null)
			this.values = null;
		else
			this.values = Arrays.asList(values);
	}
	
	public List getOptions() {
		return options;
	}
	
	public void setOptions(List opts) {
		options = opts;
	}
	
	public void addOption(Option opt) {
		if(options == null)
			options = new ArrayList();
		options.add(opt);
	}
	
	public void removeOption(Option opt) {
		if(options != null)
			options.remove(opt);
	}

	//
	//
	//
	
	public String childrenToXML(String indent, boolean inclVal) 
	{
		String tmp = super.childrenToXML(indent, inclVal);
	    if(tmp == null && getValues() == null && getOptions()==null)
	    	return null;
		StringBuffer b = new StringBuffer();
		if(tmp != null)
			b.append(tmp);
		if(getValues() != null) {
			for(int i = 0; i < getValues().size(); i++) {
				String val = (String)values.get(i);
				b.append("\n" + indent + "<"+Form.FBTAG_VALUE+">" + val + "</"+Form.FBTAG_VALUE+">");
			}
		}
		if(getOptions() != null) {
			for(int i = 0; i < getOptions().size(); i++) {
				Option opt = (Option)getOptions().get(i);
				b.append(opt.toXML(indent));
			}
		}
 		return b.toString();
	}
	
	
}
