/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.forms;

import org.w3c.dom.Node;


import java.util.*;
import java.text.*;

/**
 * <p>&lt;range&gt;</p>
 * 
 * Check that a given value is within the specified min/max range.<br> 
 * If min is not spec'ed then it means no minimum value, ditto for max.<br>
 * 
 * Parameters:<br>
 *   -error<br>
 *   -min<br>
 *   -max<br>
 *
 */
public class DateRangeValidator extends A_RangeValidator {
	
	public DateRangeValidator(Form form) {
		super(form);
	}
	
	public DateRangeValidator(Form form, Node node) {
		super(form, node);
	}
	
	public boolean validate(I_Field field) throws ParseException {
		Date minimum = null, maximum = null;
		if(getMin()!=null)
			minimum = ((DateFormat)field.getFormat().formatter()).parse(getMin());
		if(getMax() !=null)
			maximum = ((DateFormat)field.getFormat().formatter()).parse(getMax());
		if(field.getValue()==null)
			return true;
		Date val = ((DateFormat)field.getFormat().formatter()).parse(field.getValue());
		if(minimum != null && minimum.compareTo(val) == -1)
			return false;
		if(maximum != null && maximum.compareTo(val) == 1)
			return false;
		return true;
	}
	

}

