/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import java.util.Iterator;

public class DefaultFieldRender implements I_FieldRender {

	public DefaultFieldRender() {
		super();
	}

	public String render(I_Field field) throws UnknownFieldRenderException {
		// note raw fields are treated here...
		if(field instanceof ButtonField) {
			return renderButtonField((ButtonField)field);
		}
		else if(field instanceof CheckBoxField) {
			return renderCheckBoxField((CheckBoxField)field);
		}
		else if(field instanceof CheckList) {
			return renderCheckList((CheckList)field);
		}
		else if(field instanceof HiddenField) {
			return renderHiddenField((HiddenField)field);
		}
		else if(field instanceof InputField) {
			return renderInputField((InputField)field);
		}
		else if(field instanceof PopUpList) {
			return renderPopUpList((PopUpList)field);
		}
		else if(field instanceof RadioList) {
			return renderRadioList((RadioList)field);
		}
		else if(field instanceof SelectList) {
			return renderSelectList((SelectList)field);
		}
		else if(field instanceof SubmitList) {
			return renderSubmitList((SubmitList)field);
		}
		else if(field instanceof TextAreaField) {
			return renderTextAreaField((TextAreaField)field);
		}
		else if(field instanceof MessageField) {
			return renderMessageField((MessageField)field);
		}
		else {
			throw new UnknownFieldRenderException(this.getClass().getName() +  " cannot render field: " + field.tagName());
		}
	}
	
	// Renders the following types: submit|reset|button
	public String renderButtonField(ButtonField f) {
		StringBuffer b = new StringBuffer(f.getLayout().getLabelBegin());
		// render as generic button
		b.append(f.getLayout().getLabelEnd());
		b.append(f.getLayout().getFieldBegin());
		b.append("<button");
		b.append(" name=\"" + f.getName() + "\"");
		if(f.getValue() != null)
			b.append(" value=\"" + Form.escapeAttribute(f.getValue()) + "\"");
		b.append(" type=\"" + f.type() + "\"");
		if(f.isReadOnly())
			b.append(" disabled");
		//b.append(" tabindex=\"\"");
		//b.append(" accesskey=\"\"");
		//b.append(" onfocus=\"\"");
		//b.append(" onblur=\"\"");
		if(f.getIdent() != null)
			b.append(" id=\"" + f.getIdent() + "\"");
		b.append(" class=\"" + f.tagName());
		if(f.isReadOnly())
			b.append(" readonly");
		if(f.getLayout().styleName() != null)
			b.append(" " + f.getLayout().styleName());
		b.append("\"");
		if(f.getLayout().getStyle() != null)
			b.append(" style=\"" + f.getLayout().getStyle() + "\"");
		b.append(">");
		if(f.getLabel() != null)
			b.append(f.getLabel());
		b.append("</button>");			
	    b.append(f.getLayout().getPostscriptBegin()+(f.getPostscript()!=null?f.getPostscript():"")+f.getLayout().getPostscriptEnd());
		b.append(f.getLayout().getFieldEnd());
		return b.toString();
	}

	public String renderCheckBoxField(CheckBoxField f) {
		StringBuffer b = new StringBuffer(f.getLayout().getLabelBegin());
		if(f.getLabelPos() == null || !f.getLabelPos().equals("after")) {
			if(f.getLabel() != null && !f.tagName().equals(Form.FBTAG_HIDDEN)) {
				b.append("<label" + (f.getIdent() != null ? " for=\"" + f.getIdent() + "\"" : "") + ">");
				b.append(f.getLabel());
				b.append("</label>");
			}
		}
		b.append(f.getLayout().getLabelEnd());
		b.append(f.getLayout().getFieldBegin());
		b.append("<input");
		b.append(" type=\"checkbox\"");
		b.append(" name=\"" + f.getName() + "\"");
		if(f.getOnValue().equals(f.getValue())) {
			b.append(" value=\"" + Form.escapeAttribute(f.getOnValue()) + "\"");
			b.append(" checked");
		}
		if(f.isReadOnly()) {
			b.append(" disabled");
		}
		if(f.getLayout().getCols() != null)
			b.append(" size=\"" + f.getLayout().getCols() + "\"");
		//b.append(" maxlength=\"\"");
		//b.append(" src=\"\"");
		//b.append(" alt=\"\"");
		//b.append(" usemap=\"\"");
		//b.append(" ismap");
		//b.append(" tabindex=\"\"");
		//b.append(" accesskey=\"\"");
		//b.append(" onfocus=\"\"");
		//b.append(" onblur=\"\"");
		//b.append(" onselect=\"\"");
		//b.append(" onchange=\"\"");
		//b.append(" accept=\"\"");
		if(f.getIdent() != null)
			b.append(" id=\"" + f.getIdent() + "\"");
		b.append(" class=\"" + f.tagName());
		if(f.isReadOnly())
			b.append(" readonly");
		if(f.getLayout().styleName() != null)
			b.append(" " + f.getLayout().styleName());
		b.append("\"");
		if(f.getLayout().getStyle() != null)
			b.append(" style=\"" + f.getLayout().getStyle() + "\"");
		b.append(">");
		if("after".equals(f.getLabelPos())) {
			if(f.getLabel() != null && !f.tagName().equals(Form.FBTAG_HIDDEN)) {
				b.append(" <label" + (f.getIdent() != null ? " for=\"" + f.getIdent() + "\"" : "") + ">");
				b.append(f.getLabel());
				b.append("</label>");
			}
		}
	    b.append(f.getLayout().getPostscriptBegin()+(f.getPostscript()!=null?f.getPostscript():"")+f.getLayout().getPostscriptEnd());
		b.append(f.getLayout().getFieldEnd());
		return b.toString();
	}

	public String renderCheckList(CheckList f) {
		if(f.getOptions()==null || f.getOptions().size() == 0) {
			return "\n<!-- CheckList with no options -->";
		}
		StringBuffer b = new StringBuffer(f.getLayout().getLabelBegin());
		if(f.getLabel() != null) {
			b.append("<label" + (f.getIdent() != null ? " for=\"" + f.getIdent() + "\"" : "") + ">");
			b.append(f.getLabel());
			b.append("</label>");
		}
		b.append(f.getLayout().getLabelEnd());
		b.append(f.getLayout().getFieldBegin());
		Iterator i = f.getOptions().iterator();
		while(i.hasNext()) {
			Option opt = (Option)i.next();
			String optval = opt.getValue();
			b.append(f.getLayout().getOptionBegin());
			b.append("\n<input");
			b.append(" name=\"" + f.getName() + "\"");
			b.append(" type=\"checkbox\"");
			if(f.getValues() != null) {
				Iterator vals = f.getValues().iterator();
				while(vals.hasNext()) {
					String val = (String)vals.next();
					if(val != null && val.equals(optval)) {
						b.append(" checked");
						break;
					}
				}
			}
			if(f.isReadOnly())
				b.append(" disabled");
			//b.append(" size=\"\"");
			//b.append(" tabindex=\"\"");
			//b.append(" accesskey=\"\"");
			//b.append(" onfocus=\"\"");
			//b.append(" onblur=\"\"");
			//b.append(" onchange=\"\"");
			if(f.getIdent() != null)
				b.append(" id=\"" + f.getIdent() + "\"");
			b.append(" class=\"" + f.tagName());
			if(f.isReadOnly())
				b.append(" readonly");
			if(f.getLayout().styleName() != null)
				b.append(" " + f.getLayout().styleName());
			b.append("\"");
			if(f.getLayout().getStyle() != null)
				b.append(" style=\"" + f.getLayout().getStyle() + "\"");
			b.append(" value=\"" + Form.escapeAttribute(optval) + "\"");
			b.append(">");
			b.append("<span class=\"optionLabel\">" + opt.getLabel() + "</span>");
			b.append(f.getLayout().getOptionEnd());
		}
	    b.append(f.getLayout().getPostscriptBegin()+(f.getPostscript()!=null?f.getPostscript():"")+f.getLayout().getPostscriptEnd());
		b.append(f.getLayout().getFieldEnd());
		return b.toString();
	}
	
	public String renderHiddenField(HiddenField f) {
		StringBuffer b = new StringBuffer();
		b.append("<input");
		b.append(" type=\"hidden\"");
		b.append(" name=\"" + f.getName() + "\"");
		if(f.getValue() != null)
			b.append(" value=\"" + Form.escapeAttribute(f.getValue()) + "\"");
		if(f.getIdent() != null)
			b.append(" id=\"" + f.getIdent() + "\"");
		b.append(">");
		return b.toString();
	}
	
	public String renderInputField(InputField f) {
		StringBuffer b = new StringBuffer(f.getLayout().getLabelBegin());
		if(f.getLabel() != null) {
			b.append("<label" + (f.getIdent() != null ? " for=\"" + f.getIdent() + "\"" : "") + ">");
			b.append(f.getLabel());
			b.append("</label>");
		}
		b.append(f.getLayout().getLabelEnd());
		b.append(f.getLayout().getFieldBegin());
		b.append("<input");
		b.append(" type=\"" + f.type() + "\"");
		b.append(" name=\"" + f.getName() + "\"");
		if(f.getValue() != null) {
			b.append(" value=\"" + Form.escapeAttribute(f.getValue()) + "\"");
		}
		if(f.isReadOnly()) {
			if(f.type().equals("text")) {
				b.append(" readonly");
			} else {
				b.append(" disabled");
			}
		}
		
		if(f.getLayout().getCols() != null)
			b.append(" size=\"" + f.getLayout().getCols() + "\"");
		//b.append(" maxlength=\"\"");
		//b.append(" src=\"\"");
		//b.append(" alt=\"\"");
		//b.append(" usemap=\"\"");
		//b.append(" ismap");
		//b.append(" tabindex=\"\"");
		//b.append(" accesskey=\"\"");
		//b.append(" onfocus=\"\"");
		//b.append(" onblur=\"\"");
		//b.append(" onselect=\"\"");
		//b.append(" onchange=\"\"");
		//b.append(" accept=\"\"");
		if(f.getIdent() != null)
			b.append(" id=\"" + f.getIdent() + "\"");
		b.append(" class=\"" + f.tagName());
		if(f.isReadOnly())
			b.append(" readonly");
		if(f.getLayout().styleName() != null)
			b.append(" " + f.getLayout().styleName());
		b.append("\"");
		if(f.getLayout().getStyle() != null)
			b.append(" style=\"" + f.getLayout().getStyle() + "\"");
		b.append(">");
	    b.append(f.getLayout().getPostscriptBegin()+(f.getPostscript()!=null?f.getPostscript():"")+f.getLayout().getPostscriptEnd());
		b.append(f.getLayout().getFieldEnd());
		return b.toString();
	}
	
	public String renderPopUpList(PopUpList f) {
		if(f.getOptions()==null || f.getOptions().size() == 0) {
			return "\n<!-- PopUpList with no options -->";
		}
		StringBuffer b = new StringBuffer(f.getLayout().getLabelBegin());
		if(f.getLabel() != null) {
			b.append("<label" + (f.getIdent() != null ? " for=\"" + f.getIdent() + "\"" : "") + ">");
			b.append(f.getLabel());
			b.append("</label>");
		}
		b.append(f.getLayout().getLabelEnd());
		b.append(f.getLayout().getFieldBegin());
		b.append("\n<select");
		b.append(" name=\"" + f.getName() + "\"");
		if(f.isReadOnly())
			b.append(" disabled");
		//b.append(" size=\"\"");
		//b.append(" multiple=\"\"");
		//b.append(" tabindex=\"\"");
		//b.append(" accesskey=\"\"");
		//b.append(" onfocus=\"\"");
		//b.append(" onblur=\"\"");
		//b.append(" onchange=\"\"");
		if(f.getIdent() != null)
			b.append(" id=\"" + f.getIdent() + "\"");
		b.append(" class=\"" + f.tagName());
		if(f.getLayout().styleName() != null)
			b.append(" " + f.getLayout().styleName());
		b.append("\"");
		if(f.isReadOnly())
			b.append(" readonly");
		if(f.getLayout().getStyle() != null)
			b.append(" style=\"" + f.getLayout().getStyle() + "\"");
		b.append(">");
		Iterator i = f.getOptions().iterator();
		String val;
		if(f.getValue() == null && f.getIgnore() != null)
			val = f.getIgnore();
		else
			val = f.getValue();
		while(i.hasNext()) {
			Option opt = (Option)i.next();
			b.append(f.getLayout().getOptionBegin());
			b.append("\n<option");
			b.append(" label=\"" + opt.getLabel() + "\"");
			b.append(" value=\"" + Form.escapeAttribute(opt.getValue()) + "\"");
			if(opt.getValue().equals(val))
				b.append(" selected");
			b.append(">");
			b.append(opt.getLabel());
			b.append("</option>");
			b.append(f.getLayout().getOptionEnd());
		}
		b.append("\n</select>");
	    b.append(f.getLayout().getPostscriptBegin()+(f.getPostscript()!=null?f.getPostscript():"")+f.getLayout().getPostscriptEnd());
		b.append(f.getLayout().getFieldEnd());
		return b.toString();
	}
	
	public String renderRadioList(RadioList f) {
		if(f.getOptions()==null || f.getOptions().size() == 0) {
			return "\n<!-- RadioList with no options -->";
		}
		StringBuffer b = new StringBuffer(f.getLayout().getLabelBegin());
		if(f.getLabel() != null) {
			b.append("<label" + (f.getIdent() != null ? " for=\"" + f.getIdent() + "\"" : "") + ">");
			b.append(f.getLabel());
			b.append("</label>");
		}
		b.append(f.getLayout().getLabelEnd());
		b.append(f.getLayout().getFieldBegin());
		String val;
		if(f.getValue() == null && f.getIgnore() != null)
			val = f.getIgnore();
		else
			val = f.getValue();
		Iterator i = f.getOptions().iterator();
		while(i.hasNext()) {
			Option opt = (Option)i.next();
			b.append(f.getLayout().getOptionBegin());
			b.append("<input");
			b.append(" type=\"radio\"");
			b.append(" name=\"" + f.getName() + "\"");
			b.append(" value=\"" + Form.escapeAttribute(opt.getValue()) + "\"");
			if(opt.getValue().equals(val))
				b.append(" checked");
			if(f.isReadOnly())
				b.append(" disabled");
			//b.append(" readonly");
			//b.append(" size=\"\"");
			//b.append(" maxlength=\"\"");
			//b.append(" src=\"\"");
			//b.append(" alt=\"\"");
			//b.append(" usemap=\"\"");
			//b.append(" ismap");
			//b.append(" tabindex=\"\"");
			//b.append(" accesskey=\"\"");
			//b.append(" onfocus=\"\"");
			//b.append(" onblur=\"\"");
			//b.append(" onselect=\"\"");
			//b.append(" onchange=\"\"");
			//b.append(" accept=\"\"");
			b.append(" class=\"" + f.tagName());
			if(f.isReadOnly())
				b.append(" readonly");
			if(f.getLayout().styleName() != null)
				b.append(" " + f.getLayout().styleName());
			b.append("\"");
			if(f.getLayout().getStyle() != null)
				b.append(" style=\"" + f.getLayout().getStyle() + "\"");
			b.append(">");
			b.append("<span class=\"optionLabel\">" + opt.getLabel() + "</span>");
			b.append(f.getLayout().getOptionEnd());
		}
	    b.append(f.getLayout().getPostscriptBegin()+(f.getPostscript()!=null?f.getPostscript():"")+f.getLayout().getPostscriptEnd());
		b.append(f.getLayout().getFieldEnd());
		return b.toString();
	}

	public String renderSelectList(SelectList f) {
		if(f.getOptions()==null || f.getOptions().size() == 0) {
			return "\n<!-- SelectList with no options -->";
		}
		StringBuffer b = new StringBuffer(f.getLayout().getLabelBegin());
		if(f.getLabel() != null) {
			b.append("<label" + (f.getIdent() != null ? " for=\"" + f.getIdent() + "\"" : "") + ">");
			b.append(f.getLabel());
			b.append("</label>");
		}
		b.append(f.getLayout().getLabelEnd());
		b.append(f.getLayout().getFieldBegin());
		b.append("\n<select");
		b.append(" name=\"" + f.getName() + "\"");
		if(f.isReadOnly())
			b.append(" disabled");
		if(f.getLayout().getRows() != null)
			b.append(" size=\"" + f.getLayout().getRows() + "\"");
		b.append(" multiple");
		//b.append(" tabindex=\"\"");
		//b.append(" accesskey=\"\"");
		//b.append(" onfocus=\"\"");
		//b.append(" onblur=\"\"");
		//b.append(" onchange=\"\"");
		if(f.getIdent() != null)
			b.append(" id=\"" + f.getIdent() + "\"");
		b.append(" class=\"" + f.tagName());
		if(f.isReadOnly())
			b.append(" readonly");
		if(f.getLayout().styleName() != null)
			b.append(" " + f.getLayout().styleName());
		b.append("\"");
		if(f.getLayout().getStyle() != null)
			b.append(" style=\"" + f.getLayout().getStyle() + "\"");
		b.append(">");
		Iterator i = f.getOptions().iterator();
		while(i.hasNext()) {
			Option opt = (Option)i.next();
			String optval = opt.getValue();
			b.append(f.getLayout().getOptionBegin());
			b.append("\n<option");
			b.append(" label=\"" + opt.getLabel() + "\"");
			b.append(" value=\"" + Form.escapeAttribute(optval) + "\"");
			if(f.getValues() != null) {
				Iterator vals = f.getValues().iterator();
				while(vals.hasNext()) {
					String val = (String)vals.next();
					if(val.equals(optval)) {
						b.append(" selected");
						break;
					}
				}
			}
			b.append(">");
			b.append(opt.getLabel());
			b.append("</option>");
			b.append(f.getLayout().getOptionEnd());
		}
		b.append("\n</select>");
	    b.append(f.getLayout().getPostscriptBegin()+(f.getPostscript()!=null?f.getPostscript():"")+f.getLayout().getPostscriptEnd());
		b.append(f.getLayout().getFieldEnd());
		return b.toString();
	}
	
	public String renderSubmitList(SubmitList f) {
		if(f.getOptions()==null || f.getOptions().size() == 0) {
			return "\n<!-- SubmitList with no options -->";
		}
		StringBuffer b = new StringBuffer(f.getLayout().getLabelBegin());
		if(f.getLabel() != null) {
			b.append("<label" + (f.getIdent() != null ? " for=\"" + f.getIdent() + "\"" : "") + ">");
			b.append(f.getLabel());
			b.append("</label>");
		}
		b.append(f.getLayout().getLabelEnd());
		b.append(f.getLayout().getFieldBegin());
		//String val;
		//if(f.getValue() == null && f.getIgnore() != null)
		//	val = f.getIgnore();
		//else
		//	val = f.getValue();
		Iterator i = f.getOptions().iterator();
		int ndx = 0;
		while(i.hasNext()) {
			Option opt = (Option)i.next();
			b.append(f.getLayout().getOptionBegin());
			b.append("<input");
			b.append(" type=\"submit\"");
			b.append(" name=\"" + f.getName() + (ndx==0 ? "" : "." + ndx) + "\"");
			b.append(" value=\"" + Form.escapeAttribute(opt.getLabel()) + "\""); 
			//if(opt.getValue().equals(val))
			//	b.append(" checked");
			if(f.isReadOnly())
				b.append(" disabled");
			//b.append(" readonly");
			//b.append(" size=\"\"");
			//b.append(" maxlength=\"\"");
			//b.append(" src=\"\"");
			//b.append(" alt=\"\"");
			//b.append(" usemap=\"\"");
			//b.append(" ismap");
			//b.append(" tabindex=\"\"");
			//b.append(" accesskey=\"\"");
			//b.append(" onfocus=\"\"");
			//b.append(" onblur=\"\"");
			//b.append(" onselect=\"\"");
			//b.append(" onchange=\"\"");
			//b.append(" accept=\"\"");
			b.append(" class=\"" + f.tagName());
			if(f.isReadOnly())
				b.append(" readonly");
			if(f.getLayout().styleName() != null)
				b.append(" " + f.getLayout().styleName());
			b.append("\"");
			if(f.getLayout().getStyle() != null)
				b.append(" style=\"" + f.getLayout().getStyle() + "\"");
			b.append(">");
			//b.append(opt.getLabel());
			b.append(f.getLayout().getOptionEnd());
			ndx++;
		}
	    b.append(f.getLayout().getPostscriptBegin()+(f.getPostscript()!=null?f.getPostscript():"")+f.getLayout().getPostscriptEnd());
		b.append(f.getLayout().getFieldEnd());
		return b.toString();
	}
	
	public String renderTextAreaField(TextAreaField f) {
		StringBuffer b = new StringBuffer(f.getLayout().getLabelBegin());
		if(f.getLabel() != null) {
			b.append("<label" + (f.getIdent() != null ? " for=\"" + f.getIdent() + "\"" : "") + ">");
			b.append(f.getLabel());
			b.append("</label>");
		}
		b.append(f.getLayout().getLabelEnd());
		b.append(f.getLayout().getFieldBegin());
		b.append("<textarea");
		b.append(" name=\"" + f.getName() + "\"");
		if(f.getLayout().getRows() != null)
			b.append(" rows=\"" + f.getLayout().getRows() + "\"");
		if(f.getLayout().getCols() != null)
			b.append(" cols=\"" + f.getLayout().getCols() + "\"");
		//b.append(" disabled");
		if(f.isReadOnly())
			b.append(" readonly");
		//b.append(" tabindex=\"\"");
		//b.append(" accesskey=\"\"");
		//b.append(" onfocus=\"\"");
		//b.append(" onblur=\"\"");
		//b.append(" onselect=\"\"");
		//b.append(" onchange=\"\"");
		if(f.getIdent() != null)
			b.append(" id=\"" + f.getIdent() + "\"");
		b.append(" class=\"" + f.tagName());
		if(f.isReadOnly())
			b.append(" readonly");
		if(f.getLayout().styleName() != null)
			b.append(" " + f.getLayout().styleName());
		b.append("\"");
		if(f.getLayout().getStyle() != null)
			b.append(" style=\"" + f.getLayout().getStyle() + "\"");
        b.append(">");
		if(f.getValue() != null)
			b.append(f.getValue());
		b.append("</textarea>");
	    b.append(f.getLayout().getPostscriptBegin()+(f.getPostscript()!=null?f.getPostscript():"")+f.getLayout().getPostscriptEnd());
		b.append(f.getLayout().getFieldEnd());
		return b.toString();
	}
	
	public String renderMessageField(MessageField f) {
		Layout l = f.getLayout();
		int cnt = 0; // If label and/or postscript is missing, banner content should cover that part of the row as well
		String tmp = "";
		if(f.getLabel()==null)
			tmp += l.getLabelBegin()+l.getLabelEnd()+l.getFieldBegin();
		if(f.getPostscript()==null)
			tmp +=l.getPostscriptBegin()+l.getPostscriptEnd()+l.getFieldEnd();
		if(!"".equals(tmp))
			cnt = tmp.toLowerCase().split("<td").length-1;
		StringBuffer b = new StringBuffer();
		if(f.getLabel()!=null) {
		    b.append(l.getLabelBegin());
			b.append("<label>" + f.getLabel() + "</label>");
			b.append(l.getLabelEnd());
			b.append(l.getFieldBegin());
		} else {
			b.append("\n<tr><td colspan=\""+cnt+"\" class=\"Banner\">");
		}
		b.append(f.getBanner());
		if(f.getPostscript()!=null) {
			b.append(l.getPostscriptBegin()+f.getPostscript()+l.getPostscriptEnd());
			b.append(l.getFieldEnd());
		} else {
			b.append("</td></tr>");
		}
		return b.toString();
	}
}
