/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import org.w3c.dom.Node;


import java.util.regex.*;

/**
 * <p>&lt;regex&gt;</p>
 * 
 * Matches value with given pattern and returns true or false depending on the outcome.<br> 
 * If value is null or empty string it evaluates to true without matching the pattern.<br>
 * Missing or invalid pattern causes false to be returned (along with output to log)<br>
 * 
 * Parameters:<br>
 *   error<br>
 *   pattern<br>
 *
 */
public class RegExValidator extends A_Validator {
	
	private String patternStr;
	private Pattern pattern;
	
	public RegExValidator(Form form) {
		super(form);
	}
	
	public RegExValidator(Form form, Node node) {
		super(form, node);
	}
	
	public boolean validate(I_Field field) {
		if(pattern == null)
			return false;
		String value = field.getValue();
		if(value==null || value.length() == 0)
			return true;
		Matcher matcher = pattern.matcher(value);
		return matcher.matches();
	}
	
	public boolean handleChildNode(Node child, short childType, String childName)
	{
		if(super.handleChildNode(child, childType, childName))
			return true;
		if(childType == Node.ELEMENT_NODE) {
			if (childName.equals(Form.FBTAG_PATTERN)) {
				setPattern(Form.collectText(child));
				return true;
			}
		}
		return false;
	}
	
	//
	// Accessors
	//
	
	public String getPattern() {
		return this.patternStr;
	}
	
	public void setPattern(String str) {
		patternStr = str;
		if(patternStr == null) {
			pattern = null;
		} else {
			try {
				pattern = Pattern.compile(patternStr);
			} catch (PatternSyntaxException e) {
				pattern = null;
			}
		}
	}
	
	//
	//
	//
	
	public String childrenToXML(String indent) 
	{
		String tmp = super.childrenToXML(indent);
	    if(tmp == null && getPattern() == null)
	    	return null;
		StringBuffer b = new StringBuffer();
		if(tmp != null)
			b.append(tmp);

		if(getPattern() != null)
			b.append("\n" + indent + "<"+Form.FBTAG_PATTERN+">" + getPattern() + "</"+Form.FBTAG_PATTERN+">");

 		return b.toString();
	}
}

