/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.records;

import java.text.ParseException;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

public abstract class A_Record implements I_Record {

	protected RecordFactory factory;
	protected boolean isNew;
	
	public A_Record() {
		super();
		isNew = true;
	}
	
	public A_Record(RecordFactory factory) {
		this();
		init(factory);
	}

	// The rest are constructors that instantiates an object from the database
	public A_Record(RecordFactory factory, String id) throws RecordException, ParseException {
		this();
		init(factory, factory.getIdentField().fromString(id));
	}
	
	public A_Record(RecordFactory factory, Object id) throws RecordException {
		this();
		init(factory, id);	
	}
	
	public void init(RecordFactory factory) {
		this.factory = factory;
	}
	
	public void init(RecordFactory factory, Object id) throws RecordException {
		this.factory = factory;
		factory.fetchRecord(id, this);
		fetched();
	}
	
	public RecordFactory factory() {
		return factory;
	}
	
	public boolean isNew() {
		return isNew;
	}
	
	public void saved() {
		isNew = false;
	}
	
	public void fetched() {
		isNew = false;
	}
	
	public boolean hasField(String fieldName) {
		return factory.getField(fieldName) != null;
	}
	
	//
	// KVEncoding, primitives are wrapped.
	//
	public abstract void takeValueForField(RecordField field, Object value);
	public abstract Object valueForField(RecordField field);
	
	public void takeValueForKey(String key, Object value) {
		RecordField field ;
		if(key==null || (field = factory.getField(key))==null)
			throw new IllegalArgumentException("GenericRecord takeValueForKey(): Unknown field=\""+key+"\"");
		takeValueForField(field, value);
	}
	
	public Object valueForKey(String key) {
		RecordField field ;
		if(key==null || (field = factory.getField(key))==null)
			throw new IllegalArgumentException("GenericRecord valueForKey(): Unknown field=\""+key+"\"");
		return valueForField(field);
	}
	
	// Same thing but converts back and forth between string representation and objects
	public void takeStringValueForKey(String key, String value) throws ParseException {
		RecordField field;
		if(key==null || (field = factory.getField(key))==null)
			throw new IllegalArgumentException("GenericRecord takeStringValueForKey(): Unknown field=\""+key+"\"");
		takeStringValueForField(field, value);
	}
	
	public void takeStringValueForField(RecordField field, String value) throws ParseException {
		Object o = field.fromString(value);
		takeValueForField(field, o);
	}
	
	public String stringValueForKey(String key) {
		RecordField field;
		if(key==null || (field = factory.getField(key))==null)
			throw new IllegalArgumentException("GenericRecord stringValueForKey(): Unknown field=\""+key+"\"");
		return stringValueForField(field);
	}
	
	public String stringValueForField(RecordField field) {
		Object o = valueForField(field);
		return field.toString(o);
	}
	
	// These serves as convenience for handling primitive values, throws if there are class casting problems
	public boolean booleanValueForKey(String key) {
		Boolean o = (Boolean)valueForKey(key);
		return o.booleanValue();
	}
	public int intValueForKey(String key) {
		Integer o = (Integer)valueForKey(key);
		return o.intValue();
	}	
	
	//
	//
	//

	public String toString() {
		StringBuffer buf = new StringBuffer(factory.getInstanceClass()+" (isNew="+isNew()+") - fields:\n");
		Iterator keys = factory.getFields().keySet().iterator();
		while (keys.hasNext()) {
			RecordField field = (RecordField)factory.getFields().get((String)keys.next());
			Object o = this.valueForField(field);
			if(o==null)
				buf.append(field.getName()+"=NULL\n");
			else
				buf.append(field.getName()+"=["+o+"]\n");
		}
		return buf.toString();
	}
	
	// Convenience
	public Map toMap() {
		Map m = new HashMap();
		Iterator keys = factory.getFields().keySet().iterator();
		while (keys.hasNext()) {
			RecordField field = (RecordField)factory.getFields().get((String)keys.next());
			m.put(field.getName(), this.valueForField(field));
		}
		return m;
	}
}
