/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.records;

import java.util.Iterator;
import java.util.List;

/**
 * Each factory holds its own instance because the generator then can cache the statement templates
 * once compiled...
 */
public class MySQLGenerator {

	private RecordFactory factory;
	
	public MySQLGenerator(RecordFactory factory) {
		super();
		this.factory = factory;
	}

	//
	// Persistence (ignoring bagged for the moment)
	//
	
	private String selectAllSQL;
	public String selectAllSQL() {
		if(selectAllSQL==null) {
			StringBuffer buf = new StringBuffer();
			buf.append("select ");
			Iterator i = factory.getFields().keySet().iterator();
			while(i.hasNext()) {
				RecordField rf = (RecordField)factory.getField((String)i.next());
				buf.append(rf.getColumn()+(i.hasNext() ? ", " : ""));
			}
			buf.append(" from " + factory.getTableName());
			selectAllSQL = buf.toString();
			//A_OpenCms.log(I_CmsLogChannels.C_OPENCMS_INFO, "[selectAllSQL] = " + selectAllSQL);
		}
		return selectAllSQL;
	}
	public String selectAllSQL(A_Clause whereClause, Object orderBy) {
		if(whereClause == null && orderBy == null) {
			return selectAllSQL();
		} else {
			StringBuffer buf = new StringBuffer(selectAllSQL());
			if(whereClause != null) {
				buf.append(" where " + whereClause.toSQL());
			}
			if(orderBy != null) {
				buf.append(" order by ");
				if(orderBy instanceof A_OrderBy) {
					buf.append(((A_OrderBy)orderBy).toSQL());
				}
				else if(orderBy instanceof List) {
					// Must be a list of A_OrderBy's
					Iterator i = ((List)orderBy).iterator();
					while(i.hasNext()) {
						buf.append(((A_OrderBy)i.next()).toSQL());
						if(i.hasNext())
							buf.append(", ");
					}
				}
			}
			return buf.toString();
		}
	}
	private String selectIdSQL;
	public String selectIdSQL() {
		if(selectIdSQL==null) {
			selectIdSQL = selectAllSQL() + " where " + factory.getIdentField().getColumn() + " = ?";
			//A_OpenCms.log(I_CmsLogChannels.C_OPENCMS_INFO, "[selectIdSQL] = " + selectIdSQL);
		}
		return selectIdSQL;
	}
	private String selectOrderSQL;
	public String selectOrderSQL() {
		if(selectOrderSQL==null) {
			selectOrderSQL = selectAllSQL()+ " order by ? desc";
			//A_OpenCms.log(I_CmsLogChannels.C_OPENCMS_INFO, "[selectOrderSQL] = " + selectOrderSQL);
		}
		return selectOrderSQL;
	}
	private String selectCountSQL;
	public String selectCountSQL() {
		if(selectCountSQL==null) {
			selectCountSQL = "select count(*) from " + factory.getTableName();
			//A_OpenCms.log(I_CmsLogChannels.C_OPENCMS_INFO, "[selectCountSQL] = " + selectCountSQL);
		}
		return selectCountSQL;
	}
	private String updateEntrySQL;
	public String updateEntrySQL() {
		if(updateEntrySQL==null) {
			StringBuffer buf = new StringBuffer();
			buf.append("update " + factory.getTableName() + " set ");
			Iterator i = factory.getFields().keySet().iterator();
			while(i.hasNext()) {
				RecordField rf = (RecordField)factory.getField((String)i.next());
				if(rf!=factory.getIdentField())
					buf.append(rf.getColumn() + " = ?" + (i.hasNext() ? ", " : ""));
			}
			buf.append(" where " + factory.getIdentField().getColumn() + " = ?");
			updateEntrySQL = buf.toString();
			//A_OpenCms.log(I_CmsLogChannels.C_OPENCMS_INFO, "[updateEntrySQL] = " + updateEntrySQL);
		}
		return updateEntrySQL;
	}
	private String deleteEntrySQL;
	public String deleteEntrySQL() {
		if(deleteEntrySQL==null) {
			deleteEntrySQL = "delete from " + factory.getTableName() + " where " + factory.getIdentField().getName() + " = ?";
			//A_OpenCms.log(I_CmsLogChannels.C_OPENCMS_INFO, "[deleteEntrySQL] = " + deleteEntrySQL);
		}
		return deleteEntrySQL;
	}
	private String insertEntrySQL;
	public String insertEntrySQL() {
		if(insertEntrySQL==null) {
			StringBuffer buf = new StringBuffer();
			buf.append("insert into " + factory.getTableName() + " (");
			Iterator i = factory.getFields().keySet().iterator();
			while(i.hasNext()) {
				RecordField rf = (RecordField)factory.getField((String)i.next());
				buf.append(rf.getColumn() + (i.hasNext() ? ", " : ""));
			}
			buf.append(") values (");
			buf.append("?");
			int cnt = factory.getFields().size()-1;
			for(int j=0; j<cnt; j++)
				buf.append(", ?");
			buf.append(")");
			insertEntrySQL = buf.toString();
			//A_OpenCms.log(I_CmsLogChannels.C_OPENCMS_INFO, "[insertEntrySQL] = " + insertEntrySQL);
		}
		return insertEntrySQL;
	}
}
