/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.records;

import java.util.List;

public class NullClause extends A_Clause {

	public static boolean IS_NOT_NULL = false;
	public static boolean IS_NULL = true;
	
	RecordField field;
	boolean isNull;
	
	public NullClause() {
		super();
	}
	
	public NullClause(RecordFactory factory, String field, boolean isNull) {
		this(factory.getField(field), isNull);
	}

	public NullClause(RecordField field, boolean isNull) {
		this();
		init(field, isNull);
	}
	
	public void init(RecordField field, boolean isNull) {
		this.field = field;
		this.isNull = isNull;
	}
		
	// returns the number of parameters this query operand takes;
	public int cardinality() {
		return 0;
	}
	
	public Object getValue() {
		return null;
	}
	
	public List getValues() {
		return null;
	}

	// These are used to collect the fields for which the prepared statement needs to bind parameters,
	// but this one does not need a parameter hence it does not return any fields even if it is actually set.
	public RecordField getField() {
		return null;
	}
	
	public List getFields() {
		return null;
	}
	
	public String toSQL() {
		return field.getColumn() + ( isNull ? " is NULL" : " is not NULL");
	}

}
